@@ -557,7 +557,11 @@ class BatchHandler(object):
         if product:
             return product
 
-        product_key = self.config.product_key()
+        # prefer caller-provided key over configured key
+        product_key = kwargs.get('product_key')
+        if not product_key:
+            product_key = self.config.product_key()
+
         if product_key == 'upc':
 
             if entry.isdigit():
