@@ -79,17 +79,34 @@ class RattailWatcher(DataSyncWatcher):
                                   payload_type=class_name,
                                   payload_key=change.instance_uuid,
                                   deletion=change.deleted)))
+
+        # when a Product has a price relationship change (e.g. it is
+        # given or loses the "current price"), the Product record is
+        # the only thing technically updated, but the simple Rattail
+        # -> Rattail sync will use an importer which *skips* those
+        # price relation fields.  so here we explicitly add changes to
+        # the queue, which can force processing of the price relation
+        # changes.  and we definitely want these to happen *last*.
+        if 'Product' in class_names:
+            for change in [c for c in changes
+                           if c.class_name == 'Product' and not c.deleted]:
+                final.append((None, # no uuid since this need not be pruned
+                              model.DataSyncChange(
+                                  payload_type='ProductPriceAssociation',
+                                  payload_key=change.instance_uuid)))
+
         return final
 
     def prune_changes(self, keys):
         deleted = 0
         session = Session(bind=self.engine)
         for key in keys:
-            change = session.query(model.Change).get(key)
-            if change:
-                session.delete(change)
-                session.flush()
-                deleted += 1
+            if key: # note that key can sometimes be None
+                change = session.query(model.Change).get(key)
+                if change:
+                    session.delete(change)
+                    session.flush()
+                    deleted += 1
         session.commit()
         session.close()
         return deleted
@@ -142,6 +159,12 @@ class RattailConsumer(DataSyncConsumer):
             session.set_continuum_user(self.runas_username)
 
         for class_name in class_names:
+
+            # this consumer does not need to process these changes; see
+            # FromRattailToRattailBase for one which can process them
+            if class_name == 'ProductPriceAssociation':
+                continue
+
             cls = getattr(self.model, class_name)
 
             for change in [c for c in changes if c.payload_type == class_name]:
@@ -306,7 +329,43 @@ class RattailConsumer(DataSyncConsumer):
             session.delete(assignment)
 
 
-class FromRattailToRattailExportConsumer(NewDataSyncImportConsumer):
+class FromRattailToRattailBase(NewDataSyncImportConsumer):
+    """
+    Base class for Rattail -> Rattail datasync consumers
+    """
+
+    def setup(self):
+        super(FromRattailToRattailBase, self).setup()
+        self.topo_sortkey = make_topo_sortkey(self.model)
+        self.handle_price_xref = self.get_handle_price_xref()
+
+    def get_handle_price_xref(self):
+        if hasattr(self, 'handle_price_xref'):
+            return self.handle_price_xref
+
+        # TODO: this should default to True, once logic is proven
+        return self.config.getbool(
+            'rattail.datasync', 'rattail.handle_price_xref',
+            default=False)
+
+    def get_importers(self):
+        importers = super(FromRattailToRattailBase, self).get_importers()
+
+        # maybe add importer to handle product price reference changes
+        if self.get_handle_price_xref() and 'ProductPriceAssociation' not in importers:
+            importers['ProductPriceAssociation'] = self.handler.get_importer('ProductPriceAssociation')
+
+        return importers
+
+    def get_host_object(self, session, change):
+        if change.payload_type == 'ProductPriceAssociation':
+            cls = self.model.Product
+        else:
+            cls = getattr(self.model, change.payload_type)
+        return session.query(cls).get(change.payload_key)
+
+
+class FromRattailToRattailExportConsumer(FromRattailToRattailBase):
     """
     Export data changes from "local" Rattail to another
     """
@@ -317,15 +376,6 @@ class FromRattailToRattailExportConsumer(NewDataSyncImportConsumer):
         self.target_engine = self.config.rattail_engines[self.dbkey]
         self.model = self.config.get_model()
 
-    def setup(self):
-        super(FromRattailToRattailExportConsumer, self).setup()
-        self.topo_sortkey = make_topo_sortkey(self.model)
-
-        # TODO: this should default to True, once logic is proven
-        self.handle_price_xref = self.config.getbool(
-            'rattail.datasync', 'rattail.handle_price_xref',
-            default=False)
-
     def make_target_session(self):
         return Session(bind=self.target_engine)
 
@@ -345,30 +395,11 @@ class FromRattailToRattailExportConsumer(NewDataSyncImportConsumer):
         for change in changes:
             self.invoke_importer(session, change)
 
-        # if we had any Product changes, we may also want to process
-        # the "price uuid" fields for each, via a custom importer just
-        # for that purpose.  note that we do this *after* all other
-        # changes have been processed...in the hopes that ProductPrice
-        # records are already in place, and nothing will break!
-        if self.handle_price_xref:
-            product_changes = [c for c in changes
-                               if c.payload_type == 'Product' and not c.deletion]
-            if product_changes:
-                price_xref_importer = self.handler.get_importer(
-                    'ProductPriceAssociation',
-                    host_session=session, session=target_session)
-                for change in product_changes:
-                    self.process_change(session, price_xref_importer, change)
-
         target_session.commit()
         target_session.close()
 
-    def get_host_object(self, session, change):
-        cls = getattr(self.model, change.payload_type)
-        return session.query(cls).get(change.payload_key)
 
-
-class FromRattailToRattailImportConsumer(NewDataSyncImportConsumer):
+class FromRattailToRattailImportConsumer(FromRattailToRattailBase):
     """
     Import data changes from another Rattail to "local"
     """
@@ -380,10 +411,6 @@ class FromRattailToRattailImportConsumer(NewDataSyncImportConsumer):
         self.local_engine = self.config.rattail_engines[self.dbkey]
         self.model = self.config.get_model()
 
-    def setup(self):
-        super(FromRattailToRattailImportConsumer, self).setup()
-        self.topo_sortkey = make_topo_sortkey(self.model)
-
     def process_changes(self, session, changes):
         self.host_session = Session(bind=self.host_engine)
         local_session = Session(bind=self.local_engine)
@@ -413,5 +440,7 @@ class FromRattailToRattailImportConsumer(NewDataSyncImportConsumer):
         self.host_session.close()
 
     def get_host_object(self, session, change):
-        cls = getattr(self.model, change.payload_type)
-        return self.host_session.query(cls).get(change.payload_key)
+        # we use base logic to get the object, but must substitute the
+        # session to ensure we read from actual host and not local
+        return super(FromRattailToRattailImportConsumer, self).get_host_object(
+            self.host_session, change)
