@@ -223,6 +223,23 @@ class rattail_import_versions_updates(ImporterEmail):
     abstract = False
 
 
+class person_merge_request(Email):
+    """
+    Sent when a request is submitted to merge two Person records.
+    """
+    default_subject = "Person Merge Request"
+
+    def sample_data(self, request):
+        return {
+            'user_display': request.user.display_name,
+            'removing_display': "Fred Flintstone",
+            'removing_url': '#',
+            'keeping_display': "Fred Flintstone",
+            'keeping_url': '#',
+            'merge_request_url': '#',
+        }
+
+
 class upgrade_failure(Email):
     """
     Sent when an app upgrade is attempted, but ultimately failed.
