@@ -2,7 +2,7 @@
 ################################################################################
 #
 #  Rattail -- Retail Software Framework
-#  Copyright © 2010-2018 Lance Edgar
+#  Copyright © 2010-2021 Lance Edgar
 #
 #  This file is part of Rattail.
 #
@@ -78,16 +78,6 @@ class tailbone_background_color(Setting):
     name = 'background_color'
 
 
-class tailbone_legacy_mobile_enabled(Setting):
-    """
-    If set, the "legacy" (jQuery) mobile app will be exposed.  (NOTE: The main
-    web app must be *restarted* if you change this!)
-    """
-    namespace = 'tailbone'
-    name = 'legacy_mobile.enabled'
-    data_type = bool
-
-
 class rattail_single_store(Setting):
     """
     If set, the app should assume there is only one Store record, and that all
@@ -264,6 +254,56 @@ class rattail_batch_purchase_allow_expired_credits(Setting):
     data_type = bool
 
 
+class rattail_batch_purchase_allow_receiving_from_scratch(Setting):
+    """
+    Determines whether or not receiving "from scratch" is allowed.  In this
+    mode, the batch starts out empty and receiver must add product to it over
+    time.
+    """
+    group = "Purchasing / Receiving"
+    namespace = 'rattail.batch'
+    name = 'purchase.allow_receiving_from_scratch'
+    data_type = bool
+
+
+class rattail_batch_purchase_allow_receiving_from_invoice(Setting):
+    """
+    Determines whether or not receiving "from invoice" is allowed.  In this
+    mode, the user must first upload an invoice file they wish to receive
+    against.
+    """
+    group = "Purchasing / Receiving"
+    namespace = 'rattail.batch'
+    name = 'purchase.allow_receiving_from_invoice'
+    data_type = bool
+
+
+class rattail_batch_purchase_allow_receiving_from_purchase_order(Setting):
+    """
+    Determines whether or not receiving "from PO" is allowed.  In this mode,
+    the user must first select the purchase order (PO) they wish to receive
+    against.  The batch is initially populated with order quantities from the
+    PO, and user then updates (or adds) rows over time.
+    """
+    group = "Purchasing / Receiving"
+    namespace = 'rattail.batch'
+    name = 'purchase.allow_receiving_from_purchase_order'
+    data_type = bool
+
+
+class rattail_batch_purchase_allow_truck_dump_receiving(Setting):
+    """
+    Determines whether or not "truck dump" receiving is allowed.  This is a
+    rather complicated feature, where one "parent" truck dump batch is created
+    for the receiver, plus several "child" batches, one for each invoice
+    involved.
+    """
+    group = "Purchasing / Receiving"
+    namespace = 'rattail.batch'
+    name = 'purchase.allow_truck_dump_receiving'
+    data_type = bool
+
+
 class rattail_batch_purchase_mobile_images(Setting):
     """
     If set, product images will be displayed when viewing a purchasing batch row.
@@ -295,6 +335,22 @@ class rattail_batch_purchase_mobile_quick_receive_all(Setting):
     data_type = bool
 
 
+##############################
+# Reporting
+##############################
+
+class tailbone_reporting_choosing_uses_form(Setting):
+    """
+    When generating a new report, if this flag is set then you will choose the
+    report from a dropdown.  If the flag is not set then you will see all
+    reports listed on the page and you'll click the link for one.
+    """
+    group = "Reporting"
+    namespace = 'tailbone'
+    name = 'reporting.choosing_uses_form'
+    data_type = bool
+
+
 ##############################
 # Vendors
 ##############################
